@@ -1,3 +1,19 @@
+# Copyright (C) 2022 Hibiki AI Limited <info@hibiki-ai.com>
+#
+# This file is part of mirai.
+#
+# mirai is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
+#
+# mirai is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along with
+# mirai. If not, see <https://www.gnu.org/licenses/>.
+
 #' mirai: Minimalist Async Evaluation Framework for R
 #'
 #' Extremely simple and lightweight method for concurrent / parallel code
@@ -30,7 +46,6 @@ NULL
 .onLoad <- function(libname, pkgname) {
   daemons <- daemons()
   daemons <<- daemons
-  .miraisysname <<- .subset2(Sys.info(), "sysname")
   invisible()
 }
 
@@ -45,5 +60,3 @@ nanonext::is_nul_byte
 #' @export
 nanonext::unresolved
 
-.miraisysname <- NULL
-
